@@ -76,6 +76,7 @@ int epoll_add	(void *, struct event *);
 int epoll_del	(void *, struct event *);
 int epoll_recalc	(struct event_base *, void *, int);
 int epoll_dispatch	(struct event_base *, void *, struct timeval *);
+void epoll_dealloc	(void *);
 
 struct eventop epollops = {
 	"epoll",
@@ -83,7 +84,8 @@ struct eventop epollops = {
 	epoll_add,
 	epoll_del,
 	epoll_recalc,
-	epoll_dispatch
+	epoll_dispatch,
+	epoll_dealloc
 };
 
 #ifdef HAVE_SETFD
@@ -109,8 +111,14 @@ epoll_init(void)
 		return (NULL);
 
 	if (getrlimit(RLIMIT_NOFILE, &rl) == 0 &&
-	    rl.rlim_cur != RLIM_INFINITY)
-		nfiles = rl.rlim_cur;
+	    rl.rlim_cur != RLIM_INFINITY) {
+		/*
+		 * Solaris is somewhat retarded - it's important to drop
+		 * backwards compatibility when making changes.  So, don't
+		 * dare to put rl.rlim_cur here.
+		 */
+		nfiles = rl.rlim_cur - 1;
+	}
 
 	/* Initalize the kernel queue */
 
@@ -343,3 +351,19 @@ epoll_del(void *arg, struct event *ev)
 
 	return (0);
 }
+
+void
+epoll_dealloc(void *arg)
+{
+	struct epollop *epollop = arg;
+
+	if (epollop->fds)
+		free(epollop->fds);
+	if (epollop->events)
+		free(epollop->events);
+	if (epollop->epfd >= 0)
+		close(epollop->epfd);
+
+	memset(epollop, 0, sizeof(struct epollop));
+	free(epollop);
+}
