@@ -76,6 +76,7 @@ int select_add		(void *, struct event *);
 int select_del		(void *, struct event *);
 int select_recalc	(struct event_base *, void *, int);
 int select_dispatch	(struct event_base *, void *, struct timeval *);
+void select_dealloc     (void *);
 
 const struct eventop selectops = {
 	"select",
@@ -83,7 +84,8 @@ const struct eventop selectops = {
 	select_add,
 	select_del,
 	select_recalc,
-	select_dispatch
+	select_dispatch,
+	select_dealloc
 };
 
 static int select_resize(struct selectop *sop, int fdsz);
@@ -93,7 +95,7 @@ select_init(void)
 {
 	struct selectop *sop;
 
-	/* Disable kqueue when this environment variable is set */
+	/* Disable select when this environment variable is set */
 	if (getenv("EVENT_NOSELECT"))
 		return (NULL);
 
@@ -131,7 +133,7 @@ check_selectop(struct selectop *sop)
 
 }
 #else
-#define check_selectop(sop)
+#define check_selectop(sop) do {;} while (0)
 #endif
 
 /*
@@ -350,3 +352,25 @@ select_del(void *arg, struct event *ev)
 	check_selectop(sop);
 	return (0);
 }
+
+void
+select_dealloc(void *arg)
+{
+	struct selectop *sop = arg;
+
+	if (sop->event_readset_in)
+		free(sop->event_readset_in);
+	if (sop->event_writeset_in)
+		free(sop->event_writeset_in);
+	if (sop->event_readset_out)
+		free(sop->event_readset_out);
+	if (sop->event_writeset_out)
+		free(sop->event_writeset_out);
+	if (sop->event_r_by_fd)
+		free(sop->event_r_by_fd);
+	if (sop->event_w_by_fd)
+		free(sop->event_w_by_fd);
+
+	memset(sop, 0, sizeof(struct selectop));
+	free(sop);
+}
